#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "hdf.h"
#include "mfhdf.h"

#include "hdiff.h"
#include "dumplib.h"
char *progname;

static void
usage()
{
#define USAGE   "\
  [-g]              Compare global attributes only\n\
  [-s]              Compare SD local attributes only\n\
  [-d]              Compare SD data only\n\
  [-D]              Compare Vdata data only\n\
  [-v var1[,...]]   Comapre SD data on variable(s) <var1>,... only\n\
  [-u var1[,...]]   Comapre vdata on variable(s) <var1>,... only\n\
  [-e count]        Print difference up to count number for each variable\n\
  [-t limit]        Print difference when it is greater than limit\n\
  file1             File name of the first input HDF file\n\
  file2             File name of the second input HDF file\n"

    (void) fprintf(stderr,
       "%s [-g] [-s] [-d] [-D] [-v ...] [-e cnt] [-t limit] file1 file2\n%s",
		   progname,
		   USAGE);
    exit(EXIT_FAILURE);
}


int
main(argc, argv)
int argc;
char *argv[];
{
    extern int optind;
    extern int opterr;
    extern char *optarg;
    static struct fspec fspec =	/* defaults, overridden on command line */
      {
	  1,		/* compare global attributes */
	  1,		/* compare SD local attributes */
	  1,		/* compare SD data */
          1,            /* compare Vdata */
          0,            /* no limit on the difference to be printed */
        0.0,            /* exact equal */
	  0,		/* if -v specified, number of variables */
	  0,		/* if -v specified, list of variable names */
	  0,		/* if -u specified, number of variables */
	  0		/* if -u specified, list of variable names */
	  };
    int c;
    int i;
    int32 sdid1, sdid2;
    int32 fid1, fid2;

    opterr = 1;
    progname = argv[0];

    if (argc == 1)
      usage();

    while ((c = getopt(argc, argv, "gsdDe:t:v:u:")) != EOF)
      switch(c) {
	case 'g':		/* global attributes only */
	  fspec.ga = 1;
          fspec.sa = 0;
          fspec.sd = 0;
          fspec.vd = 0;
	  break;
	case 's':		/* SD local attributes only */
	  fspec.ga = 0;
          fspec.sa = 1;
          fspec.sd = 0;
          fspec.vd = 0;
	  break;
	case 'd':		/* SD data only */
	  fspec.ga = 0;
          fspec.sa = 0;
          fspec.sd = 1;
          fspec.vd = 0;
	  break;
	case 'D':		/* SD data only */
	  fspec.ga = 0;
          fspec.sa = 0;
          fspec.sd = 0;
          fspec.vd = 1;
	  break;
        case 'e':		/* max no. of difference to be printed */
          fspec.max_err_cnt = atoi(optarg);
          break;
        case 't':		/* range of difference to be printed */
          fspec.err_limit = atof(optarg);
          break;
	case 'v':		/* variable names */
	  /* make list of names of variables specified */
	  make_vars (optarg, &fspec, 1);
	  break;
	case 'u':		/* variable names */
	  /* make list of names of variables specified */
	  make_vars (optarg, &fspec, 2);
	  break;
	case '?':
	  usage();
	  break;
      }

    argv = argv + optind;

    sdid1 = SDstart(argv[0], DFACC_RDONLY); 
    if (sdid1 == -1)
    {
      error("SDstart failed on %s", argv[0]);
      return EXIT_SUCCESS;
    }

    sdid2 = SDstart(argv[1], DFACC_RDONLY);
    if (sdid2 == -1)
    {
      SDend(sdid1);
      error("SDstart failed on %s", argv[1]);
      return EXIT_SUCCESS;
    }

    if (fspec.ga == 1) 
      gattr_diff(sdid1, sdid2);

    if (fspec.sa == 1)
      sdattr_diff(sdid1, sdid2);

    if (fspec.sd == 1)
      sddata_diff(sdid1, sdid2, fspec);

    SDend(sdid1);
    SDend(sdid2);

    
    fid1 = Hopen(argv[0], DFACC_READ, 0); 
    if (fid1 == -1)
    {
      error("Hopen failed on %s", argv[0]);
      return EXIT_SUCCESS;
    }

    fid2 = Hopen(argv[1], DFACC_READ, 0); 
    if (fid2 == -1)
    {
      error("Hopen failed on %s", argv[1]);
      return EXIT_SUCCESS;
    }

    if (fspec.vd == 1)
    {
      vdata_diff(fid2, fid1, fspec, 0, 1);
      vdata_diff(fid1, fid2, fspec, 1, 0);
    }

    Hclose(fid1);
    Hclose(fid2);

    return EXIT_SUCCESS;
}
